module HEX_SWITCH  (
   input          OSC_50 , 
   input  [3:0]   SW , 
	input  [7:0]   Temperature        ,  
   input  [7:0]   Humidity           ,	
	input  [15:0]  LIGHT_SENSOR_ADC0 ,
	input  [15:0]  LIGHT_SENSOR_ADC1 ,
 	input  [15:0]  Accelerometer_X   ,
	input  [15:0]  Accelerometer_Y   ,
	input  [15:0]  Accelerometer_Z   ,
	input  [15:0]  Gyroscope_X       ,
	input  [15:0]  Gyroscope_Y       ,
	input  [15:0]  Gyroscope_Z       ,
	input  [15:0]  Magnetometer_X    ,
	input  [15:0]  Magnetometer_Y    ,
	input  [15:0]  Magnetometer_Z    ,		
	input          RH_TEMP_DRDY_n , 
	input   [2:0]  POWERUP , 
	input          HM_TR, 
 	output  [7:0]	HEX0,
	output  [7:0]	HEX1,
	output  [2:0]	LEDR,
	output  [15:0] RESULT ,	
	output         SP_CLK,
	output  [7:0]  BCD_T,
   output  [7:0]  BCD_H	,
   output  [3:0]  DIG0,
   output  [3:0]  DIG1
); 

//-- Sample Clock   ---
CLOCKMEM  rf1( .CLK   (OSC_50 ) , .CLK_FREQ ( 500*20 ), .CK_1HZ   (SP_CLK )  ) ;


//--LED 	
assign  LEDR =   { RH_TEMP_DRDY_n , POWERUP , HM_TR }; 


//-- Temperature HEX TO BCD
HEX2BCD  h00(
 .iCLK_50 ( OSC_50      ),
 .iHEX    ( Temperature[7:0] ), 
 .oBCD    ( BCD_T      )  
);
//-- Humidity HEX TO BCD
HEX2BCD  h01(
 .iCLK_50 ( OSC_50       ),
 .iHEX    ( Humidity[7:0] ), 
 .oBCD    ( BCD_H      )  
 
);

//----------------2 HEX VIEW -------
//SW:0   temperature (positive) in decimal     
//SW:1   humidity (positive) in decimal     
//SW:2   LIGHT SENSOR ADC0 (DATA0) MSB 16-bit hexadecimal (positive)
//SW:3   LIGHT SENSOR ADC1 (DATA1) MSB 16-bit hexadecimal (positive)
//SW:4   THREE-AXIS MEMS ACCELEROMETER X MSB 16-bit hexadecimal (positive/negative)
//SW:5   THREE-AXIS MEMS ACCELEROMETER Y MSB 16-bit hexadecimal (positive/negative)
//SW:6   THREE-AXIS MEMS ACCELEROMETER Z MSB 16-bit hexadecimal (positive/negative)
//SW:7   THREE-AXIS MEMS GYROSCOPE     X MSB 16-bit hexadecimal (positive/negative)
//SW:8   THREE-AXIS MEMS GYROSCOPE     Y MSB 16-bit hexadecimal (positive/negative)
//SW:9   THREE-AXIS MEMS GYROSCOPE     Z MSB 16-bit hexadecimal (positive/negative)
//SW:10  THREE-AXIS MEMS MAGNETOMETER  X MSB 16-bit hexadecimal (positive/negative)
//SW:11  THREE-AXIS MEMS MAGNETOMETER  Y MSB 16-bit hexadecimal (positive/negative)
//SW:12  THREE-AXIS MEMS MAGNETOMETER  Z MSB 16-bit hexadecimal (positive/negative)
//SW:13~ 0
//---------------------------------	
//---DATA SELECTION 
assign  DIG0  = (
(SW==0 )?BCD_T [3:0]:(
(SW==1 )?BCD_H [3:0]:(
(SW==2 )?LIGHT_SENSOR_ADC0 [11:8]:(
(SW==3 )?LIGHT_SENSOR_ADC1 [11:8]:(
(SW==4 )?Accelerometer_X   [11:8]:(
(SW==5 )?Accelerometer_Y   [11:8]:(
(SW==6 )?Accelerometer_Z   [11:8]:(
(SW==7 )?Gyroscope_X       [11:8]:(
(SW==8 )?Gyroscope_Y       [11:8]:(
(SW==9 )?Gyroscope_Z       [11:8]:(
(SW==10)?Magnetometer_X    [11:8]:(
(SW==11)?Magnetometer_Y    [11:8]:(
(SW==12)?Magnetometer_Z    [11:8]:0
)))))))))))));


assign  DIG1  = (
(SW==0 )?BCD_T  [7:4]:(
(SW==1 )?BCD_H  [7:4]:(
(SW==2 )?LIGHT_SENSOR_ADC0 [15:12]:(
(SW==3 )?LIGHT_SENSOR_ADC1 [15:12]:(
(SW==4 )?Accelerometer_X   [15:12]:(
(SW==5 )?Accelerometer_Y   [15:12]:(
(SW==6 )?Accelerometer_Z   [15:12]:(
(SW==7 )?Gyroscope_X       [15:12]:(
(SW==8 )?Gyroscope_Y       [15:12]:(
(SW==9 )?Gyroscope_Z       [15:12]:(
(SW==10)?Magnetometer_X    [15:12]:(
(SW==11)?Magnetometer_Y    [15:12]:(
(SW==12)?Magnetometer_Z    [15:12]:0
)))))))))))));

//--HEX OUTPUT
SEG7_LUT_V	 h20(	 .iDIG (DIG0) , .oSEG (HEX0) ) ; 
SEG7_LUT_V	 h21(	 .iDIG (DIG1) , .oSEG (HEX1) ) ; 


//-----------Signaltape VIEW-----------
//SW:0   Temperature     (positive) in decimal   
//SW:1   Humidity        (positive) in decimal  
//SW:2   LIGHT SENSOR ADC0 (DATA0)   (positive) in decimal
//SW:3   LIGHT SENSOR ADC1 (DATA1)   (positive) in decimal
//SW:4   Accelerometer X  (positive/negative) in decimal
//SW:5   Accelerometer Y  (positive/negative) in decimal
//SW:6   Accelerometer Z  (positive/negative) in decimal
//SW:7   Gyroscope X  (positive/negative) in decimal
//SW:8   Gyroscope Y  (positive/negative) in decimal
//SW:9   Gyroscope Z  (positive/negative) in decimal
//SW:10  Magnetometer X  (positive/negative) in decimal
//SW:11  Magnetometer Y  (positive/negative) in decimal
//SW:12  Magnetometer Z  (positive/negative) in decimal
//SW:13~ 0
//-------------------------------------

assign RESULT = ( 
   (SW==0 )? Temperature      :( 
   (SW==1 )? Humidity         :( 
	(SW==2 )? LIGHT_SENSOR_ADC0:(
	(SW==3 )? LIGHT_SENSOR_ADC1:(
 	(SW==4 )? Accelerometer_X  :(
	(SW==5 )? Accelerometer_Y  :(
	(SW==6 )? Accelerometer_Z  :(
	(SW==7 )? Gyroscope_X      :(
	(SW==8 )? Gyroscope_Y      :(
	(SW==9 )? Gyroscope_Z      :(
	(SW==10)? Magnetometer_X   :(
	(SW==11)? Magnetometer_Y   :(
	(SW==12)? Magnetometer_Z   : 0
	)))))))))))));
  

endmodule 
 